require_relative '../lib/sass-buddy.rb'
require 'tmpdir'
require 'pry'

RSpec.describe SassBuddy do
  it 'compiles scss with given headers' do
    styles = Tempfile.new("style.scss")
    begin
      styles.write(".hi { color: $var1; }")
      styles.rewind

      subject = described_class.new({
        file: styles.path,
        additions: {
          "var1": "14a2d4"
        }
      })

      expect(subject.compile).to eq(".hi {\n  color: #14a2d4; }\n")
    ensure
      styles.close
      styles.unlink
    end
  end

  it 'compiles scss from obscure paths' do
    begin
      tmp_dir = File.expand_path "#{Dir.pwd}/#{Time.now.to_i}#{rand(1000)}/"
      FileUtils.mkdir_p tmp_dir

      styles = Tempfile.new("style.scss")
      external = Tempfile.new(['external', '.scss'], tmp_dir)
      FileUtils.mv external.path, "#{tmp_dir}/external.scss"

      styles.write("@import 'external'")
      external.write(".hi { color: $var1; }")

      external.rewind
      styles.rewind

      subject = described_class.new({
        file: styles.path,
        additions: {
          "var1": "14a2d4"
        },
        require_paths: {
          "external": tmp_dir + '/'
        }
      })

      expect(subject.compile).to eq(".hi {\n  color: #14a2d4; }\n")
    ensure
      styles.close
      styles.unlink
      external.close
      external.unlink
      FileUtils.rm_rf( tmp_dir ) if File.exists?( tmp_dir )
    end
  end

  it 'revves according to a manifest' do
    styles = Tempfile.new("style.scss")
    manifest = Tempfile.new("manifest.json")
    begin
      manifest.write("{\"images/fart.jpg\":\"images/fart-1234.jpg\"}")
      manifest.rewind
      styles.write(".hi { color: $var1; background-image: image-url('images/fart.jpg'); }")
      styles.rewind

      subject = described_class.new({
        file: styles.path,
        additions: {
          "var1": "14a2d4"
        },
        rev_manifest: manifest.path
      })

      expect(subject.compile).to eq(".hi {\n  color: #14a2d4;\n  background-image: image-url(\"images/fart-1234.jpg\"); }\n")
    ensure
      styles.close
      styles.unlink
      manifest.close
      manifest.unlink
    end
  end

  it 'compiles strings and contains them' do
    string = 'h1 { color: blue }'
    container = '#content'

    subject = described_class.new({
      string: string,
      container: container
    })

    expect(subject.compile).to eq("#content h1 {\n  color: blue; }\n")
  end

  it 'creates html tags' do
    string = 'h1 { color: blue }'
    container = '#content'

    subject = described_class.new({
      string: string,
      container: container
    })

    expect(subject.compile_html).to eq("<style>#content h1 {\n  color: blue; }\n</style>")
  end

  it 'compiles sass' do
    string = "$color: red

=my-border($color)
  border: 1px solid $color

body
  background: $color
  +my-border(green)"

    subject = described_class.new({
      string: string,
      sass: true
    })

    expect(subject.compile).to eq("body {\n  background: red;\n  border: 1px solid green; }\n")
  end

  it 'returns false for bad css' do
    string = 'h1 { color: blue } adi8q3d9n'
    container = '#content'

    subject = described_class.new({
      string: string,
      container: container
    })

    expect(subject.compile_html).to eq(nil)
  end
end